# -*- encoding: utf-8 -*-
#
# Copyright 2013 IBM Corp.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo.config import cfg

CONF = cfg.CONF

opts = [
    cfg.StrOpt('api_audit_map',
               default='/etc/pycadf/api_audit_map.conf',
               help='File containing mapping for api paths and '
                    'service endpoints'),
]
CONF.register_opts(opts, group='audit')
